
#############################################################################
# Copyright (C) Labomedia May 2012
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franproplin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
#############################################################################

import GameLogic as gl
import socket
from OSC3.OSC import decodeOSC
from time import sleep
# init global var
ip = gl.ip
port = gl.port_in
buffer_size = 1024

stack =[] # player order for display in blender, a little bit complexed, but not complicated
data = [] # osc data with modif along the script
players_data = [] # list of list

# gl.master_or_slave=true if I'm master
master_position = -1 # master position in player_list

# Get controller and owner
controller = gl.getCurrentController()
owner = controller.owner
# Get all scenes
scenes = gl.getSceneList()
# For all scenes, all objets, all prop
# Get bat and ball
for scen in scenes:
    for obj in scen.objects:
        if str(obj) == "2_bat_1" :
            bat_1 = obj
        if str(obj) == "ball2":
            ball2 = obj

def main():
    data = get_osc()
    i_quit(data) # I quit if /quit message in data
    players_data = set_players_data_with_data(data)
    # players_data = [ ['10.0.0.5',0,0,8,0,7,1,5,1], 9 items
    # ip, master, number, score0, score1, batx, baty, ballx, bally

    # stack is complicated
    global master_position
    stack, master_position = create_players_stack(players_data)

    a = len(stack) # players number
    if a > 0:
        owner["level"] = a # set level with bricks
        owner["Text"] = "Players number: " + str(a)

    print_usefull_var(("Got:", players_data, stack, master_position))

    # Set ball
    if not gl.master_or_slave:
        if a > 1:
            # disable ball rigid body with prop master=False in ball
            ball2["master"] = False
            set_ball_position(players_data, stack, master_position)

    # Always set bat of other player
    # ok only with 2 players
    set_bat_position(players_data, stack, master_position)
    # score
    set_score(players_data, stack, master_position)

def set_score(players_data, stack, master_position):
    # only with 2 players
    scen.objects["Score_1"]["Text"] = int(scen.objects["2_bat_0"]["score0"])
    scen.objects["Score_2"]["Text"] = int(scen.objects["2_bat_0"]["score1"])

    if len(stack) == 2:
        if not gl.master_or_slave:
            scen.objects["Score_1"]["Text"] = players_data[master_position][2]
            scen.objects["Score_2"]["Text"] = players_data[master_position][3]

def set_bat_position(players_data, stack, master_position):
    # only with 2 players
    if len(stack) == 2:
        other = stack[1]
        for p_list in players_data:
            if p_list[2] == other:
                bat_1.localPosition = [-p_list[5], -p_list[6], 1]

def set_ball_position(players_data, stack, master_position):
    #print("ball ok")
    ball2.localPosition = [-players_data[master_position][7], -players_data[master_position][8], 1]

def get_osc():
    global data
    # Connect Blender
    if not gl.connected :
        # if socket exist and not close
        try:
            gl.sock.close()
        except:
            pass

        gl.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        try:
            gl.sock.bind((ip, port))
            gl.sock.setblocking(0)
            gl.sock.settimeout(0.005)
            gl.sock.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, buffer_size)
            print('Get OSC on IP={} Port={} Buffer={}'.format(ip, port, buffer_size))
            gl.connected = True
        except:
            print('No connected to IP={} Port={}'.format(ip, port))
            pass

    # If Blender connected
    if gl.connected :
        try:
            raw_data = gl.sock.recv(buffer_size)
            #print(raw_data)
            data = decodeOSC(raw_data)
            #print(data)
        except socket.error:
            pass
    return data

def disconnect():
    gl.sock.close()

def chunks(l, n):
    # coupe en n listes de liste de len l
    return [l[i:i+n] for i in range(0, len(l), n)]

def set_players_data_with_data(data):
    # a list of data of all players
    global players_data
    if '/for_all_players' in data :
        data = data[2:]
        if len(data) % 9 == 0 : # verif pour qu il ne manque pas de data
            players_data = chunks(data, 9) # division en sous liste
    return players_data

def create_players_stack(players_data):
    # stack is a list of player number, first is my number
    # then ascending from my number
    # 1, 5, 6, 9 where I am 5 : stack = 5, 9, 1, 6
    player_list = []
    global master_position
    global stack
    for i in range(len(players_data)):
        player_list.append(players_data[i][2]) # add player number
        if players_data[i][1] == 1: # if this player is master
            master_position = players_data.index(players_data[i]) #  master position of the list in the global list
        if players_data[i][0] == gl.local_ip: # where is my ip ?
            gl.player = players_data[i][2] # this is my player number with my ip
            # I'm master or slave ?
            if players_data[i][1] == 1:
                gl.master_or_slave = True # My physics motor is used for all players
            else :
                gl.master_or_slave = False # I am not master, set ball position
    # I sort in ascending order [5, 1, 9, 6] with my number first
    if len(player_list) > 0:
        player_list.sort() # ascending order
        # where is my player number in list
        ix = player_list.index(gl.player)
        # I cut the list after my number
        a = player_list[ix:]
        # I cut the list before my number
        b = player_list[:ix]
        # The stack
        stack = a + b # stack = 5, 9, 1, 6
    return stack, master_position

def print_usefull_var(usefull_var):
    # print every 60 frames
    gl.tp += 1
    if gl.tp == 60:
        print(usefull_var)
        gl.tp = 0

def i_quit(data):
    # very bad atchtung
    global players_data
    if '/quit' in data :
        owner["quit"] = True
        gl.sock.close()

main()

##print(scenes)
##scenes[0].suspend()
##sleep(10)
##scenes[0].restart()
